/* USAGE
 *
 * var json_client = jQuery.JR(url)
 * 
 * Where 'url' is the URL of the JSON-RPC server.
 *
 * To a remote method use:
 * json_client.<remote_method>([params]);
 *
 * Please note, that if the name of the remote method contains special symbols like / or dot
 * that special symbol will be transliterated to underscore (_) and the name of the method will
 * be changed. As example if listed method is: 'BOOKS/list'
 * You may call it using: json_client.BOOKS_list();
 *
 * See also: http://plugins.jquery.com/project/jsonRPC2
 */
jQuery.JR = function(_url) {
	/* Create an object that can be used to make JSON RPC calls. */
	return new (function(_url){
		/* Self reference variable to be used all over the place */
		var self = this;
	
		this.url = _url; // target URL
		this.sequence = 1; // request ID
		this.error = false; // See if we're in an error condition.

		/* Do an ajax request to the server and build our object. */
		jQuery.ajax({
			async: false,
			contentType: 'application/json; charset=UTF-8',
			type: 'post',
			processData: false,
			dataType: 'json',
			url: self.url,
			cache: false,
			data    : '{"jsonrpc": "2.0","method":"system.listMethods"}',
			success : function(data) {
				/* On success, let's build some callback methods. */
				for(var i in data.result) attachFunction(data.result[i]);
				self['multi'] = multi;
				if (typeof console != 'undefined' && typeof console.log == 'function') console.log('jquery.jr.js: Init Remote Functions', data);
			}
		});
		
		function attachFunction(key) {
			self[key.replace(/\W/g,'_')] = function(){
				var params = new Array();
				for(var i = 0; i < arguments.length; i++) params.push(arguments[i]);

				var id = (self.sequence++);
				var reply = [];


				/* AJAX away! */
				jQuery.ajax({
					async: false,
					contentType: 'application/json; charset=UTF-8',
					type: 'post',
					processData: false,
					dataType: 'json',
					url: self.url,
					cache: false,
					data: JSON.stringify({jsonrpc: '2.0',method: key,params: params,id: id}),
					error: function(req,stat,err){
						self.error = true;
						self.error_message = stat;
						self.error_request = req;
						reply = jQuery.parseJSON(req.responseText);
					},
					success: function(inp){
						reply = inp;
					}
				});

				return reply;
			}
		}
		
		function multi() {
			var requests = new Array();
			for(var i = 0; i < arguments.length; i++){
				var arg = arguments[i];
				var id = (self.sequence++);
				var method, params;
				if((typeof arg) == 'string'){
					method = arg;
					params= [];
				}else{
					method = arg[0];
					params = arg.splice(1, 1);
				}
				requests[i] = {jsonrpc: '2.0',method: method.replace(/_/i,'/') ,params: params,id: id};
			}

     		var reply = [];
			/* AJAX away! */
			jQuery.ajax({
				async: false,
				contentType: 'application/json; charset=UTF-8',
				type: 'post',
				processData: false,
				dataType: 'json',
				url: self.url,
				cache: false,
				data: JSON.stringify(requests),
				error: function(req,stat,err){
					self.error = true;
					self.error_message = stat;
					self.error_request = req;
					reply = jQuery.parseJSON(req.responseText);
				},
				success: function(inp){
					reply = inp;
				}
			});

			return reply;
		}
		
		function getParams(func){
			var b = func.indexOf('(');
			if(b == -1) return [];
			var e = func.indexOf(')');
			if(e == -1 || e < b) return [];
			b = b + 1;
			if (b < e){
				var arr = func.substring(b, e).split(","); 
				return arr;
			}	
			return [];
		}
		
		return this;

	})(_url);
	
	
	
};
